#include <math.h>
#include <string.h>
#include <common.h>

#include <hzr_set_error.h>
#include <hzr_qextd.h>
#include "setcoe_setup_indices.h"
#include "setcoe_obs_loop.h"
#include "setcoe_calc_scaling.h"

#define SETCOE_DEFS
#include "setcoe.h"

void setcoe(void)
 
     /**************************************************************
   SETUP FOR CONSERVATION OF EVENTS
								    
   PURPOSE
     1.  TO SET UP MODEL FOR CONSERVATION OF EVENTS
 
     2.  TO STORE SHAPING FUNCTIONS IF ALL SHAPING PARAMETERS FOR A
         PHASE ARE FIXED
 
     3.  TO DETERMINE WHICH SCALING PARAMETER WILL BE ESTIMATED
         NONITERATIVELY
 
     4.  TO PERFORM CONSERVATION OF EVENTS COMPUTATIONS FOR THE
         ITERATIVE SCHEME
 
     5.  TO ITERATIVELY UPDATE EXPLICITLY SOLVED SCALE FUNCTION
 
     6.  TO PERFORM A MORE LIMITED UPDATING PRIOR TO A STEPWISE
         PROCEDURE STEP
 
  __________________________________________________________________
 
           C O N S E R V A T I O N   O F   E V E N T S
 
  __________________________________________________________________
 
     MALCOLM E. TURNER, JR., PH.D. HAS SHOWN (PROOF OF THEOREM IS
     AVAILABLE UPON REQUEST) THAT FOR SEMI-INFINITE, POSITIVE
     DISTRIBUTIONS, 0 <= T < INFINITY, T CONTINUOUS, AND RIGHT
     CENSORED DATA THAT THE SUM OVER ALL OBSERVATIONS OF INDIVIDUAL
     CUMULATIVE HAZARD FUNCTIONS AT T(I)--THE DURATION OF FOLLOWUP--
     EQUALS THE NUMBER OF OBSERVED EVENTS UNDER THE FOLLOWING
     SET OF RESTRICTIONS (SEE M.E.T. NOTES OF FEBRUARY 14, 1985):
 
     IF AND ONLY IF
 
                    -G(THETA)*D(T)
       F(T)  = 1 - E
 
                -G(THETA)*D(T)
       S(T)  = E
 
       CF(T) = G(THETA)*D(T)
 
       HF(T) = G(THETA)*DDOT(T)
 
     WHERE
 
       F(T) IS THE CUMULATIVE MORTALITY FUNCTION
       S(T) IS THE CUMULATIVE SURVIVAL FUNCTION
       CF(T) IS THE CUMULATIVE HAZARD FUNCTION
       HF(T) IS THE HAZARD FUNCTION
       G(THETA) IS A PARAMETRIC SCALING FUNCTION OF THE ONE PARAMETER
         THETA
       D(T) IS A PARAMETRIC FUNCTION OF TIME T (AND POSSIBLY OTHER
         PARAMETERS)
       DDOT(T) IS THE TIME DERIVATIVE OF D(T)
 
     THEN
 
       THE LOG LIKELIHOOD FUNCTION IS
 
                      _                     _
                 NOBS|                       |
         LLIKE = SUM |E(I)*LOG(HF(T)) - CF(T)|
                 I=1 |_                     _|
 
                      _                                            _
                     |     _                        _               |
                 NOBS|    |                          |              |
               = SUM |E(I)|LOG(G(THETA))+LOG(DDOT(T))|-G(THETA)*D(T)|
                 I=1 |    |_                        _|              |
                     |_                                            _|
 
       HERE
 
         E(I) IS THE EVENT INDICATOR (0 IF NO EVENT, 1 IF AN EVENT)
         T IS UNDERSTOOD AS T(I)
 
     NOW TAKE THE PARTIAL DERIVATIVE OF LLIKE WITH RESPECT TO THETA:
 
                       _                                          _
                      |                                            |
       D(LLIKE)   NOBS|     D(G(THETA))     1      D(G(THETA))     |
       -------- = SUM |E(I)*-----------*-------- - -----------*D(T)|
       D(THETA)   I=1 |      D(THETA)   G(THETA)    D(THETA)       |
                      |_                                          _|
 
 
     AT MAXIMUM LIKELIHOOD ESTIMATES FOR ALL PARAMETERS, SET THE 
     ABOVE TO ZERO.
 
                              D(THETA)
     NOW MULTIPLY THROUGH BY -----------*G(THETA) :
                             D(G(THETA))
 
                _                  _         _          _
           NOBS|                    |   NOBS|            |
       0 = SUM |E(I) - G(THETA)*D(T)| = SUM |E(I) - CF(T)|
           I=1 |_                  _|   I=1 |_          _|
 
 
       WHERE THETA IS ACTUALLY THETA-HAT AND CF(T) IS ALSO CF(T)-HAT.
 
     FINALLY, REARRANGE TO OBSERVE THAT:
            _    _         _     _
       NOBS|      |   NOBS|       |
       SUM | E(I) | = SUM | CF(T) |
       I=1 |_    _|   I=1 |_     _|
 
       WHERE CF(T) IS, AGAIN, CF(T)-HAT.
 
     FURTHER:  IF THE DATA ARE INTEVAL CENSORED, THEN THE SAME
     MANIPULATIONS CAN BE USED TO SHOW THAT FOR INTERVAL CENSORED
     DATA, GIVEN THE ABOVE DISTRIBUTION MODEL, THERE IS CONSERVATION
     OF EVENTS.  THE RESULT IS AS FOLLOWS:
 
     LET
 
       CT    = THE TIME AT WHICH INTERVAL CENSORING BEGINS FOR
               INTERVAL CENSORED DATA.  NOTE:  IT ENDS AT T; THAT
               IS, THE EVENT OCCURS BETWEEN CT AND T, CT<T.
 
     THEN
 
        NOBS          NOBS
        SUM C1(I)  +  SUM C3(I)  =
        I=1           I=1
 
 
          NOBS
          SUM (C1(I) + C2(I) + C3(I))*CF(T)
          I=1
 
 
 
     YET FURTHER:  IF THE DATA CONTAIN LEFT CENSORING, THERE IS STILL
     CONSERVATION OF EVENTS AS FOLLOWS:
 
     LET
 
       ST    = THE TIME AT WHICH LEFT CENSORING BEGINS
               NOTE:  ST IS SMALLER THAN T.
 
     THEN
 
        NOBS          NOBS
        SUM C1(I)  +  SUM C3(I)  =
        I=1           I=1
 
 
          NOBS
       +  SUM (C1(I) + C2(I) + C3(I))*(CF(T) - CF(ST))
          I=1
 
 
     IF THE EVENT IS WEIGHTED, SUCH AS BY A COST, OR BY SEVERITY OF
     A MORBID EVENT, THEN ANY OF THE THREE FOLLOWING ASSUMPTIONS 
     LEADS TO A FOMULATION OF A WEIGHTED (POSSIBLY REPEATING) EVENT 
     AND ITS MEAN "COST."  
     1) USING FISHER'S (UNPROVEN FROM FIRST PRINCIPLES, BUT
     "RESONABLE") WEIGHTING ASSUMPTION, SIMPLY WEIGHT EACH PIECE OF
     THE LOG LIKELIHOOD FUNCTION (A MULTIPLICATION IN  THE LOG
     LIKELIHOOD DOMAIN, AN EXPONENT IN THE LIKELIHOOD DOMAIN).  
     2) USING SO-CALLED FUZZY THEORY, LET THE USUAL LIKELIHOOD 
     EXPONENT TAKE ON VALUES OTHER THAN 0 OR 1.  THIS CAUSES THE
     USUAL LL FORMULATION TO TAKE ON A SOMEWHAT COMPLEX STRUCTURE, 
     BUT IN THE END IT REDUCES TO SIMPLE WEIGHTING OF ALL LOG 
     LIKELIHOOD COMPONENTS.
     3) FROM FIRST PRINCIPLES, ESTABLISH TWO COMPLETELY SEPARATE
     COMPONENTS:  AN EXPONENT THAT TAKES ON THE VALUE OF ONLY 0 OR 1
     AND WHICH IS USED SIMPLY FOR SELECTION, THEREFORE, OF HAZARD
     PIECES (THIS FORMULATION LEADS TO THE COUNTS C1, C2, AND C3 AS
     ABOVE), AND A COMPONENT THAT SPECIFIES WEIGHTING, WHICH IS
     PLACED AS A MULTIPLIER OF THE ENTIRE LOG LIKELIHOOD EXPRESSION
     (ONLY ONE COMPONENT OF WHICH APPLIES TO EACH UNIQUE TYPE OF
     INFORMATION FROM THE ABOVE).  THIS SCHEME LEADS DIRECTLY TO THE
     FORMULATION DERIVED FROM THE TWO OTHER SETS OF ASSUMPTIONS.
 
     LET
 
       W    = WEIGHT OF EVENT WITH KNOWN TIME T

     THEN
 
        NOBS                NOBS
        SUM C1(I)*W(I)  +  SUM C3(I)*W(I)
        I=1                 I=1
 
 
          NOBS
       +  SUM (C1(I) + C2(I) + C3(I))*(CF(T) - CF(ST))
          I=1
 
 
     WHERE
 
       C1(I) = COUNT OF NUMBER OF EVENTS AT TIME=T
 
       C2(I) = COUNT OF CENSORED INDIVIDUALS AT TIME=T
 
       C3(I) = COUNT OF INTERVAL CENSORED INDIVIDUALS AT TIME=1 AND
               AN IDENTICAL CENSORING INTERVAL BEGINNING AT TIME=CT
                    _                             _
                3  |                               |
       CF(T) = SUM |MU(J;B(J),CONCOM)*G(T;PARMS(J))|
               J=1 |_                             _|
 
 
       CONCOM   =  VECTOR OF CONCOMMITENT INFORMATION OF LENGTH P
 
       B(J)     =  PARAMETER VECTOR OF LENGTH P+1
 
       PARMS(1) = VECTOR OF EARLY PHASE PARAMETERS (DELTA,RHO,NU,M)
 
       PARMS(2) = 1
 
       PARMS(3) = VECTOR LATE PHASE PARAMETERS (TAU,GAMMA,ALPHA,ETA)
 
       MU(J;B(J),CONCOM) = SCALING PARAMETRIC FUNCTION
 
       G(T;PARMS(J)) = SHAPING FUNCTION FOR CUMULATIVE HAZARD
 
     NOTE:  1.  SEE SETG1 AND SETG3 FOR FORM OF SHAPING FUNCTION
 
            2.  G(2) = T.
 
            3.  FOR WEIGHTED CASE, RATHER THAN C1(I) + C3(I) WE HAVE
                C1(I)*WEIGHT(I) + C3(I)*WEIGHT(I)
 
  ___________________________________________________________________
 
 
                  I M P L I C A T I O N S
 
  ___________________________________________________________________
 
 
     1.  SINCE EVENTS ARE CONSERVED, ONE SCALING PARAMETER CAN BE
         SOLVED FOR EXPLICITLY, GIVEN THE REMAINING MODEL.  THUS
         ONE SCALING PARAMETER IS REMOVED FROM THE MAINLINE
         OPTIMIZATION ROUTINE PROCESS.
 
     2.  NO MATTER THE INITIAL GUESSES FOR OTHER MODEL PARAMETERS,
         BY USING THE CONSERVATION OF EVENTS THEOREM, WE CAN ALWAYS
         SCALE THE MODEL TO BE IN A REASONABLE RANGE.  THIS SHOULD
         AID THE OPTIMIZATION OF THE SHAPING PARAMETERS.
 
 
   DESCRIPTION OF PARAMETERS FOR SETCOE
     INPUT
       OBS    - REAL*8  NOBS*(P+7) VECTOR OF OBSERVATIONS
       THETA  - REAL*8  N-VECTOR OF ALL MODEL PARAMETERS
       STATUS - INTEGER N-VECTOR FLAGS FOR ACTIVE PARAMETERS
 
     OUTPUT
       LNG1   - REAL*8  NOBS-VECTOR OF LOGE(G1) FIXED SHAPING FUNCTION
       LNG1CT - REAL*8  NOBS-VECTOR OF LOGE(G1) FIXED SHAPING FUNCTION
                        AT CT (START OF INTERVAL CENSORING)
       LNG1ST - REAL*8  NOBS-VECTOR OF LOGE(G1) FIXED SHAPING FUNCTION
                        AT ST (START OF INTERVAL)
       LNSG1  - REAL*8  NOBS-VECTOR OF LOGE(SG1) FIXED SHAPING FUNCTION
       LNG2   - REAL*8  NOBS-VECTOR OF LOGE(T)
       LNG2CT - REAL*8  NOBS-VECTOR OF LOGE(CT) FIXED SHAPING FUNCTION
                        (START OF INTERVAL CENSORING)
       LNG2ST - REAL*8  NOBS-VECTOR OF LOGE(ST) FIXED SHAPING FUNCTION
                        (START OF INTERVAL)
       LNG3   - REAL*8  NOBS-VECTOR OF LOGE(G3) FIXED SHAPING FUNCTION
       LNG3CT - REAL*8  NOBS-VECTOR OF LOGE(G3) FIXED SHAPING FUNCTION
                        AT CT (START OF INTERVAL CENSORING)
       LNG3ST - REAL*8  NOBS-VECTOR OF LOGE(G3) FIXED SHAPING FUNCTION
                        AT ST (START OF INTERVAL)
       LNSG3  - REAL*8  NOBS-VECTOR OF LOGE(SG1) FIXED SHAPING FUNCTION
       LNMU1  - REAL*8  NOBS-VECTOR OF LOGE(MU1), PHASE 1 INTERCEPT
       LNMU2  - REAL*8  NOBS-VECTOR OF LOGE(MU1), PHASE 2 INTERCEPT
       LNMU3  - REAL*8  NOBS-VECTOR OF LOGE(MU1), PHASE 3 INTERCEPT
 
     WORKING
       INDXX  - INTEGER IVAR-VECTOR FOR STORING INDICES FOR X
       INDXP  - INTEGER IVAR-VECTOR FOR STORING CORRESPONDING PHASES
       INDXT  - INTEGER IVAR-VECTOR FOR STORING CORRESPONDING THETAS
 
   DESCRIPTION OF OTHER PARAMETERS USED BY OR CREATED BY SUBROUTINE
     INPUT
       MODE   - INTEGER DESCRIBES THE MODE OF COMPUTATIONS
                        0 - STANDARD MODE
                        1 - APPLY CONSERVATION ONLY INITIALLY
                        2 - CALCULATE, BUT NEVER APPLY CONSERVATION
       FIXG1  - LOGICAL IF .TRUE. EARLY SHAPING FUNCTION IS FIXED
       FIXG3  - LOGICAL IF .TRUE. LATE SHAPING FUNCTION IS FIXED
       IVAR   - INTEGER NUMBER OF VARIABLES IN SCALING FUNCTIONS
       NOBS   - INTEGER NUMBER OF OBSERVATIONS IN DATA SET
       P      - INTEGER NUMBER OF UNIQUE CONCOMITANT VARIABLES
       PHASE  - INTEGER 3-VECTOR, =1 IF PHASE IS PRESENT
       NTHETA - INTEGER NUMBER OF ALL MODEL PARAMETERS
 
     OUTPUT
       FACTOR - REAL*8  FACTOR BY WHICH CURRENT MODEL FAILS TO
                        CONSERVE EVENTS
       FIXMU  - INTEGER PHASE (1, 2, OR 3) WHOSE INTERCEPT SHOULD
                        NOT BE DETERMINED BY OPTIMIZATION ROUTINE
       ERROR  - INTEGER ERROR FLAG
                        0 - NO ERROR
                        1 - EXPONENTIAL OVERFLOW
                        2 - VARIABLE OUTSIDE OF BOUNDS (EG. SIGN)
       ERRFLG - CHARACTER*12 ERROR MESSAGE
 
   DESCRIPTION OF PARAMETERS FOR ENTRY CONSTP
     INPUT
       NONE, HOWEVER IT DEPENDS ON INITIAL SETUP AND CHANGE IN
       THETA, THE STARTING PARAMETER ESTIMATES AND CURRENT VALUES
       OF SHAPING AND INTERCEPT PARAMETERS
 
     OUTPUT
       NONE, HOWEVER IT UPDATES THE VALUE OF FACTOR AND IVAR
 
   DESCRIPTION OF PARAMETERS FOR ENTRY CONSRV
     INPUT
       NONE, HOWEVER IT DEPENDS ON INITIAL SETUP AND CHANGE IN
       THETA, THE STARTING PARAMETER ESTIMATES AND CURRENT VALUES
       OF SHAPING AND INTERCEPT PARAMETERS
 
     OUTPUT
       NONE, HOWEVER IT UPDATES THE VALUE OF FACTOR AND SETS UP
       TEMPORARY STORAGE TO FACILITATE CALCULATION OF THE OBJECTIVE
       FUNCTION
 
   REMARKS
 
     1.  IF A CONSTANT PHASE ONLY IS PRESENT WITHOUT CONCOMMITANT
         INFORMATION, THE VALUE GIVEN BY THE CONSERVATION OF EVENTS
         SETUP ROUTINE TO ITS INTERCEPT IS, IN FACT, THE MAXIMUM
         LIKELIHOOD VALUE AND FURTHER OPTIMIZATION IS UNNECESSARY.
         IF A SINGLE NON-CONSTANT HAZARD PHASE IS PRESENT BUT ALL
         SHAPING PARAMETERS ARE FIXED, THE VALUE GIVEN BY THE
         CONSERVATION OF EVENTS SETUP ROUTINE TO ITS INTERCEPT IS
         ALSO THE MAXIMUM LIKELIHOOD VALUE AND FURTHER OPTIMIZATION
         IS UNNECESSARY.
 
     2.  SUBROUTINE SETOBJ, THROUGH THE ENTRY DHAZRD, CALLS THE ENTRY
         POINTS TO DO THE ITERATIVE UPDATING OF THE EXPLICITLY SOLVED
         FOR SCALING PARAMETER.  DURING THIS PROCESS THE WORKING
         ARRAYS LNMU1, LNMU2, AND/OR LNMU3 ARE USED TO AVOID
         RECALCULATIONS NECESSARY IN SUBSEQUENT EVALUATION OF THE
         OBJECTIVE FUNCTION.  IN ADDITION, IF THE SHAPING PARAMETERS
         ARE NOT FIXED, THE STORAGE AREAS FOR THE LOGARITHMS OF THE
         G'S ARE USED AS INTERMEDIATE STORAGE.
 
     3.  IT IS IMPORTANT THAT THE EXPLICITLY CALCULATED PARAMETER
         BE USED IN EVALUATING THE OBJECTIVE FUNCTION.  THUS, THE
         OBJECTIVE FUNCTION EVALUATION SHOULD FIRST DETERMINE
         THIS PARAMETER, THEN EVALUATE THE OBJECTIVE FUNCTION.
 
     4.  INITIALLY, ALL INPUT SCALING PARAMETERS ARE ADJUSTED BY
         AN EQUAL PROPORTION TO CONSERVE EVENTS BEFORE ITERATIVE
         OPTIMIZATION IS DONE.
 
     5.  IF IN A VARIABLE SELECTION MODE, THEN PRIOR TO THE
         OPTIMIZATION CALL, SETCON IS CALLED.  THEN WITHIN THE
         STEPWISE LOOP, CALL CONSTP JUST BEFORE RETURNING TO
         THE OPTIMIZATION.
 
   SUBPROGRAMS REQUIRED
     DLGAOB - CALCULATE LOGE(A/B)
     DLG1   - CALCULATE LOGE(G1(T)), RETURNS LG1 AND LSG1
     DLG3   - CALCULATE LOGE(G3(T)), RETURNS LG3 AND LSG3
 
   METHOD
     APPLICATION OF CONSERVATION OF EVENTS.
       WE PROPOSE THAT OUR MODEL BE FORMULATED INITIALLY AS:
 
         FACTOR*CF(T)
 
       WHERE CF(T) IS DEFINED AS ABOVE.  THIS IMPLIES THAT FACTOR
       WILL BE DETERMINED AND USED TO UPDATE THE VALUES OF ALL
       INTERCEPT PRIOR TO THE ITERATIVE OPTIMIZATION PROCEDURE.
       AS OPTIMIZATION PROCEDES, THIS FACTOR SHOULD TEND TOWARD
       A VALUE OF ONE, AND THE APPROXIMATION BELOW WILL HOLD
       EXACTLY.
 
     APPROXIMATION
 
        NOBS                NOBS
        SUM C1(I)*W(I)  +  SUM C3(I)*W(I)  APPROXIMATELY EQUALS:
        I=1                 I=1
 
               _                                        _
              |                                          |
              |NOBS                                      |
       FACTOR*|SUM (C1(I)+C2(I)+C3(I))*(CF'(T) - CF'(ST))|
              |I=1                                       |
              |_                                        _|
 
 
       WHERE
 
         CF' AND S' INDICATE THE CALCULATION OF CUMULATIVE HAZARD
         AND SURVIVAL FUNCTIONS RESPECTIVELY AT CURRENT MODEL
         PARAMETERS, UNADJUSTED FOR FACTOR (CONSERVATION OF EVENTS).
 
     THIS LINEAR FUNCTION CAN NOW BE SOLVED DIRECTLY FOR FACTOR.
 
     USING THIS APPROACH, ALL INTERCEPTS ARE UPDATED INITIALLY BY
     FACTOR PRIOR TO ITERATIVE OPTIMIZATION.
 
     AT EACH ITERATION, WE WILL CALCULATE THE AMOUNT OF CUMULATIVE
     HAZARD CONTRIBUTED BY THE PHASE WHOSE SCALING PARAMETER IS
     SOLVED EXPLICITLY.  WE ALSO WILL CALCULATE THE TOTAL CUMULATIVE
     HAZARD ACCOUNTED FOR CURRENTLY BY THE ENTIRE MODEL.  ANY
     DISCREPENCY BETWEEN TOTAL CUMULATIVE HAZARD AND THE KNOWN
     NUMBER OF EVENTS IS THEN ABSORBED BY THE EXPLICITLY DEFINED
     SCALE PARAMETER.  IN IMPLEMENTING THIS, WE AGAIN MERELY FIND
     A MULTIPLICATIVE FACTOR WHICH WILL TEND TOWARD 1.0 AS THE
     ITERATIVE PROCESS CONTINUES.  THE DANGER WITH THIS APPROACH
     IS THAT THE NUMBER OF EVENTS IN THE SELECTED PHASE COULD BE
     CALCULATED AS A NEGATIVE NUMBER, AN IMPOSSIBILITY, SINCE
     THIS WOULD MEAN THAT THE OTHER PHASES HAVE MORE EVENTS
     ACCOUNTED FOR THAN HAVE OCCURRED.  WE BELIEVE SUCH AN ANOMALY
     OF OPTIMIZATION WILL BE RARE IF WE UPDATE THE EXPLICIT
     SCALE FACTOR AT EVERY FUNCTION EVALUATION.  OTHERWISE AN
     ERROR MESSAGE IS GIVEN.
 
   IMPLEMENTATION
     E.H. BLACKSTONE, MD     OCTOBER, 1985
     CHECKED FOR POSSIBLE OPTIMIZATION OF NUMERICAL COMPUTAIONS BY
     C.R. KATHOLI,PH.D       MARCH, 1986
 
   REVISIONS
     MARCH 25,1986 - ADD EXTENDED ARITHMETIC TO MAIN SUMMATIONS
     MARCH 26,1986 - VARIABLE DIMENSIONED ARRAYS ADDED TO CALLING
                     SEQUENCE
     APRIL 7,1986 - PERFORM CONSERVATION OF EVENTS PRIOR TO EVALUATING
                    OBJECTIVE FUNCTION.  THIS REQUIRED TEMPORARILY
                    STORING INTERMEDIATE PRODUCTS TO AVOID UNNECESSARY
                    RECALCULATIONS IN DHAZRD.
     MAY 25,1987  - CORRECT INTERVAL CENSORING LOOP
     JUNE 22,1987 - CORRECT INTERVAL CENSORING LOOP IN ENTRY CONSRV
     JULY 1,1987  - REMOVE REDUNDANT CALCULATIONS BY USING INXEX ARRAYS
     APR 13,1991  - ADD WEIGHTED EVENTS CODING
     SEP 25,1991  - CHANGE TO NELSON FORMULATION FOR SIMPLICITY
     Dec 1993     - Converted to C-version 4.0(0)
     Feb 22,1994  - Corrected weighting scheme (M. E. Turner)
     Aug  3,2012  - Corrected weighting scheme (J. Ehrlinger)
 
   VERIFICATION
     NOVEMBER 29, 1985
     COMBINATIONS OF RIGHT AND INTERVAL CENSORED DATA IN ALL PHASES
     TESTED AGAINST DATA SETS RUN USING ORIGINAL KATHOLI PROGRAM
     AND AGAINST CALCULATIONS MADE BY SAS.
 
     OCTOBER, 1991:  TESTING OF REVISIONS IN LIKELIHOOD DEFINITION,
     LEFT CENSORING, AND NEW FORMULATION FOR INTERVAL CENSORED EVENTS
     TESTED AGAINST ORIGINAL PROGRAM.
 
***********************************************************************/
 
{
  /* double sumchz,lfactr;*/
  double phmax;
  /*long i; */
  long j;
 
  if(HZRstr.mode<0 || HZRstr.mode>2) {
    hzr_set_error("(SETCOE900)",2);
    return;
  }
 
  /* INITIALIZE */
  if(Common.p<0) {
    hzr_set_error("(SETCOE910)",2);
    return;
  }
 
  Common.errorno = 0;
  CLEAR(Common.errflg);
 
  HZRstr.pp1 = Common.p+1;
  HZRstr.pp7 = Common.p+7;
  HZRstr.mp = -Common.p;
  HZRstr.pj1 = 8 + HZRstr.mp;

  G.entry = SETCOE;
 
  SETCOE_setup_indices();
 
  Setcoe.qsumcz = QEXTD(ZERO);
  Setcoe.sc1pc3 = ZERO;
  for(j=1; j<=3; j++)
    Setcoe.sumcf[j] = ZERO;
 
  HZRstr.nfncts++;
 
  /**
     IF THERE IS NO CONCOMMITANT INFORMATION, THE SCALING FACTORS FOR
     EACH INDIVIDUAL REMAIN THE SAME.
  **/
  if(Common.p==0)
    for(j=1; j<=3; j++)
      if(Common.phase[j]==1)
	G.lmu[j] = Common.theta[j+8-1];
 
  for(G.I=0; G.I<Common.Nobs; G.I++)
    if(!SETCOE_obs_loop())
      return;

  SETCOE_calc_scaling();
 
  /**
     DETERMINE WHICH INTERCEPT TO ELIMINATE
 
     DO THIS BY DETERMINING WHICH PHASE HAS THE LARGEST CUMULATIVE
     HAZARD.
  **/
  phmax = ZERO;
  for(j=1; j<=3; j++)
    if(Setcoe.sumcf[j]>phmax) {
      phmax = Setcoe.sumcf[j];
      HZRstr.fixmu = j;
    }
 
  /**
     NOW TURN OFF STATUS FLAG OF THE SCALING PARAMETER WE WILL FIX
     EXCEPT FOR APPLYING THE CONSERVATION OF EVENTS PRINCIPLE
  **/
  if(HZRstr.mode==0)
    Common.status[HZRstr.pj1+HZRstr.pp1*HZRstr.fixmu-1] = 0;
 
  return;
}
 

 

